import logging
import sys

# Simple terminal-logger instance 
def setup_logger(level:str="INFO"):

    custom_logger = logging.getLogger("fwl."+__name__)
    custom_logger.handlers.clear()

    level = str(level).strip().upper()
    if level not in ["INFO", "DEBUG", "ERROR", "WARNING"]:
        raise ValueError("Invalid log level '%s'"%level)
    level_code = logging.getLevelName(level)

    # Add terminal output to logger
    sh = logging.StreamHandler(sys.stdout)
    sh.setLevel(level_code)
    custom_logger.addHandler(sh)
    custom_logger.setLevel(level_code)

    # Capture unhandled exceptions
    # https://stackoverflow.com/a/16993115
    def handle_exception(exc_type, exc_value, exc_traceback):
        if issubclass(exc_type, KeyboardInterrupt):
            custom_logger.error("KeyboardInterrupt")
            sys.__excepthook__(exc_type, exc_value, exc_traceback)
            return
        custom_logger.critical("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))
    sys.excepthook = handle_exception
    
    return custom_logger



import mors
import pytest
from numpy.testing import assert_allclose

TEST_DATA = (
    ((0.128, 45.2, 8.5e1),(0.21263373, 1.68612614e+31, 1.73275380e+28)),
    ((1.113, 17.7, 3.2e3),(1.16581827, 6.81769926e+33, 7.94292986e+28)),
    ((0.995, 1.005, 1.0e4),(1.48904952e+00, 7.80667546e+33, 3.22933432e+28)),
    ((1.000, 1.00, 1.0e4),(1.52583342e+00, 8.13191736e+33, 3.38245392e+28)),
)

@pytest.mark.parametrize("inp,expected", TEST_DATA)
def test_spada_FE(inp, expected):

    mors.DownloadEvolutionTracks('Spada')
    params = mors.NewParams()
    params['TimeIntegrationMethod'] = 'ForwardEuler'
    star = mors.Star(Mstar=inp[0], Omega=inp[1], params = params)
    ret = (
         star.Value(inp[2], 'Rstar'),
         star.Value(inp[2], 'Lbol'),
         star.Value(inp[2], 'Leuv'),
         )

    assert_allclose(ret, expected, rtol=1e-6, atol=0)



import logging
import os
import subprocess
from pathlib import Path

import platformdirs
from osfclient.api import OSF

import logging
log = logging.getLogger("fwl."+__name__)

FWL_DATA_DIR = Path(os.environ.get('FWL_DATA', platformdirs.user_data_dir('fwl_data')))

log.info(f'FWL data location: {FWL_DATA_DIR}')

#project ID of the stellar evolution tracks folder in the OSF
project_id = '9u3fb'

def download_folder(*, storage, folders: list[str], data_dir: Path):
    """
    Download a specific folder in the OSF repository

    Inputs :
        - storage : OSF storage name
        - folders : folder names to download
        - data_dir : local repository where data are saved
    """
    for file in storage.files:
        for folder in folders:
            if not file.path[1:].startswith(folder):
                continue
            parts = file.path.split('/')[1:]
            target = Path(data_dir, *parts)
            target.parent.mkdir(parents=True, exist_ok=True)
            log.info(f'Downloading {file.path}...')
            with open(target, 'wb') as f:
                file.write_to(f)
            break


def GetFWLData() -> Path:
    """
    Get path to FWL data directory on the disk
    """
    return Path(FWL_DATA_DIR).absolute()

def DownloadEvolutionTracks(fname=""):
    """
    Download evolution track data

    Inputs :
        - fname (optional) :    folder name, "Spada" or "Baraffe"
                                if not provided download both
    """

    #Create stellar evolution tracks data repository if not existing
    data_dir = os.path.join(GetFWLData() , "stellar_evolution_tracks")
    os.makedirs(data_dir, exist_ok=True)

    #Link with OSF project repository
    osf = OSF()
    project = osf.project(project_id)
    storage = project.storage('osfstorage')

    #If no folder name specified download both Spada and Baraffe
    if not fname:
        folder_list = ("Spada", "Baraffe")
    elif fname in ("Spada", "Baraffe"):
        folder_list = [fname]
    else:
        raise ValueError(f"Unrecognised folder name: {fname}")

    folders = [folder for folder in folder_list if not os.path.exists(os.path.join(data_dir, folder))]

    if folders:
        log.info(f"Downloading MORS evolution tracks to {data_dir}")
        download_folder(storage=storage, folders=folders, data_dir=data_dir)

    if "Spada" in folders:
        #Unzip Spada evolution tracks
        wrk_dir = os.getcwd()
        os.chdir(os.path.join(data_dir , "Spada"))
        subprocess.call( ['tar','xvfz', 'fs255_grid.tar.gz'] )
        subprocess.call( ['rm','-f', 'fs255_grid.tar.gz'] )
        os.chdir(wrk_dir)

    return
